@@ -31,7 +31,6 @@ GENERATOR_OPTS = [
 
 RULE_OPTS = [
     cfg.MultiStrOpt('namespace',
-                    required=True,
                     help='Option namespace(s) under "oslo.policy.policies" in '
                          'which to query for options.'),
     cfg.StrOpt('format',
@@ -42,7 +41,6 @@ RULE_OPTS = [
 
 ENFORCER_OPTS = [
     cfg.StrOpt('namespace',
-               required=True,
                help='Option namespace under "oslo.policy.enforcer" in '
                     'which to look for a policy.Enforcer.'),
 ]
@@ -193,12 +191,19 @@ def _format_rule_default_yaml(default, include_help=True):
              'check_str': default.check_str,
              'reason': default.deprecated_reason}
 
-        text = (
-            '%(text)s%(deprecated_text)s\n"%(old_name)s": "rule:%(name)s"\n'
-        ) % {'text': text,
-             'deprecated_text': _format_help_text(deprecated_text),
-             'old_name': default.deprecated_rule.name,
-             'name': default.name}
+        if default.name != default.deprecated_rule.name:
+            text = (
+                '%(text)s%(deprecated_text)s\n"%(old_name)s": "rule:%(name)s"'
+                '\n'
+            ) % {'text': text,
+                 'deprecated_text': _format_help_text(deprecated_text),
+                 'old_name': default.deprecated_rule.name,
+                 'name': default.name}
+        else:
+            text = (
+                '%(text)s%(deprecated_text)s\n'
+            ) % {'text': text,
+                 'deprecated_text': _format_help_text(deprecated_text)}
 
     return text
 
@@ -308,6 +313,11 @@ def _list_redundant(namespace):
     in a policy file and operators should consider removing them.
     """
     enforcer = _get_enforcer(namespace)
+    # NOTE(bnemec): We don't want to see policy deprecation warnings in the
+    # output of this tool. They tend to overwhelm the output that the user
+    # actually cares about, and checking for deprecations isn't the purpose of
+    # this tool.
+    enforcer.suppress_deprecation_warnings = True
     # Ensure that files have been parsed
     enforcer.load_rules()
 
@@ -322,21 +332,36 @@ def on_load_failure_callback(*args, **kwargs):
     raise
 
 
-def generate_sample(args=None):
+def _check_for_namespace_opt(conf):
+    # NOTE(bnemec): This opt is required, but due to lp#1849518 we need to
+    # make it optional while our consumers migrate to the new method of
+    # parsing cli args. Making the arg itself optional and explicitly checking
+    # for it in the tools will allow us to migrate projects without breaking
+    # anything. Once everyone has migrated, we can make the arg required again
+    # and remove this check.
+    if conf.namespace is None:
+        raise cfg.RequiredOptError('namespace', 'DEFAULT')
+
+
+def generate_sample(args=None, conf=None):
     logging.basicConfig(level=logging.WARN)
-    conf = cfg.ConfigOpts()
+    # Allow the caller to pass in a local conf object for unit testing
+    if conf is None:
+        conf = cfg.CONF
     conf.register_cli_opts(GENERATOR_OPTS + RULE_OPTS)
     conf.register_opts(GENERATOR_OPTS + RULE_OPTS)
     conf(args)
+    _check_for_namespace_opt(conf)
     _generate_sample(conf.namespace, conf.output_file, conf.format)
 
 
 def generate_policy(args=None):
     logging.basicConfig(level=logging.WARN)
-    conf = cfg.ConfigOpts()
+    conf = cfg.CONF
     conf.register_cli_opts(GENERATOR_OPTS + ENFORCER_OPTS)
     conf.register_opts(GENERATOR_OPTS + ENFORCER_OPTS)
     conf(args)
+    _check_for_namespace_opt(conf)
     _generate_policy(conf.namespace, conf.output_file)
 
 
@@ -355,12 +380,15 @@ def _upgrade_policies(policies, default_policies):
                           'new_name': rule_default.name})
 
 
-def upgrade_policy(args=None):
+def upgrade_policy(args=None, conf=None):
     logging.basicConfig(level=logging.WARN)
-    conf = cfg.ConfigOpts()
+    # Allow the caller to pass in a local conf object for unit testing
+    if conf is None:
+        conf = cfg.CONF
     conf.register_cli_opts(GENERATOR_OPTS + RULE_OPTS + UPGRADE_OPTS)
     conf.register_opts(GENERATOR_OPTS + RULE_OPTS + UPGRADE_OPTS)
     conf(args)
+    _check_for_namespace_opt(conf)
     with open(conf.policy, 'r') as input_data:
         policies = policy.parse_file_contents(input_data.read())
     default_policies = get_policies_dict(conf.namespace)
@@ -384,8 +412,9 @@ def upgrade_policy(args=None):
 
 def list_redundant(args=None):
     logging.basicConfig(level=logging.WARN)
-    conf = cfg.ConfigOpts()
+    conf = cfg.CONF
     conf.register_cli_opts(ENFORCER_OPTS)
     conf.register_opts(ENFORCER_OPTS)
     conf(args)
+    _check_for_namespace_opt(conf)
     _list_redundant(conf.namespace)
