@@ -227,6 +227,49 @@ class GenerateSampleYAMLTestCase(base.PolicyBaseTestCase):
 # favor of "foo:create_bar":"role:fizz". foo:post_bar is being removed
 # in favor of foo:create_bar
 "foo:post_bar": "rule:foo:create_bar"
+'''
+        stdout = self._capture_stdout()
+        with mock.patch('stevedore.named.NamedExtensionManager',
+                        return_value=test_mgr) as mock_ext_mgr:
+            generator._generate_sample(['rules'], output_file=None)
+            mock_ext_mgr.assert_called_once_with(
+                'oslo.policy.policies', names=['rules'],
+                on_load_failure_callback=generator.on_load_failure_callback,
+                invoke_on_load=True
+            )
+        self.assertEqual(expected, stdout.getvalue())
+
+    def test_deprecated_policies_with_same_name(self):
+        deprecated_rule = policy.DeprecatedRule(
+            name='foo:create_bar',
+            check_str='role:old'
+        )
+        new_rule = policy.RuleDefault(
+            name='foo:create_bar',
+            check_str='role:fizz',
+            description='Create a bar.',
+            deprecated_rule=deprecated_rule,
+            deprecated_reason=(
+                'role:fizz is a more sane default for foo:create_bar'
+            ),
+            deprecated_since='N'
+        )
+        opts = {'rules': [new_rule]}
+
+        extensions = []
+        for name, opts in opts.items():
+            ext = stevedore.extension.Extension(name=name, entry_point=None,
+                                                plugin=None, obj=opts)
+            extensions.append(ext)
+        test_mgr = stevedore.named.NamedExtensionManager.make_test_instance(
+            extensions=extensions, namespace=['rules'])
+
+        expected = '''# Create a bar.
+#"foo:create_bar": "role:fizz"
+
+# DEPRECATED "foo:create_bar":"role:old" has been deprecated since N
+# in favor of "foo:create_bar":"role:fizz". role:fizz is a more sane
+# default for foo:create_bar
 '''
         stdout = self._capture_stdout()
         with mock.patch('stevedore.named.NamedExtensionManager',
@@ -457,8 +500,9 @@ class GeneratorRaiseErrorTestCase(testtools.TestCase):
     def test_generator_call_with_no_arguments_raises_error(self):
         testargs = ['oslopolicy-sample-generator']
         with mock.patch('sys.argv', testargs):
+            local_conf = cfg.ConfigOpts()
             self.assertRaises(cfg.RequiredOptError, generator.generate_sample,
-                              [])
+                              [], local_conf)
 
 
 class GeneratePolicyTestCase(base.PolicyBaseTestCase):
@@ -521,7 +565,8 @@ class ListRedundantTestCase(base.PolicyBaseTestCase):
     def setUp(self):
         super(ListRedundantTestCase, self).setUp()
 
-    def test_matched_rules(self):
+    @mock.patch('warnings.warn')
+    def test_matched_rules(self, mock_warn):
         extensions = []
         for name, opts in OPTS.items():
             ext = stevedore.extension.Extension(name=name, entry_point=None,
@@ -544,7 +589,13 @@ class ListRedundantTestCase(base.PolicyBaseTestCase):
         enforcer.register_default(
             policy.RuleDefault('owner', 'project_id:%(project_id)s'))
         # register a new opt
-        enforcer.register_default(policy.RuleDefault('foo', 'role:foo'))
+        deprecated_rule = policy.DeprecatedRule('old_foo', 'role:bar')
+        enforcer.register_default(
+            policy.RuleDefault('foo', 'role:foo',
+                               deprecated_rule=deprecated_rule,
+                               deprecated_reason='reason',
+                               deprecated_since='T')
+            )
 
         # Mock out stevedore to return the configured enforcer
         ext = stevedore.extension.Extension(name='testing', entry_point=None,
@@ -575,6 +626,9 @@ class ListRedundantTestCase(base.PolicyBaseTestCase):
         self.assertEqual('"owner"', opt1[0])
         self.assertEqual('"project_id:%(project_id)s"', opt1[1])
 
+        self.assertFalse(mock_warn.called,
+                         'Deprecation warnings not suppressed.')
+
 
 class UpgradePolicyTestCase(base.PolicyBaseTestCase):
     def setUp(self):
@@ -602,6 +656,8 @@ class UpgradePolicyTestCase(base.PolicyBaseTestCase):
                                             plugin=None,
                                             obj=[self.new_policy])
         self.extensions.append(ext)
+        # Just used for cli opt parsing
+        self.local_conf = cfg.ConfigOpts()
 
     def test_upgrade_policy_json_file(self):
         test_mgr = stevedore.named.NamedExtensionManager.make_test_instance(
@@ -616,7 +672,7 @@ class UpgradePolicyTestCase(base.PolicyBaseTestCase):
                         self.get_config_file_fullname('new_policy.json'),
                         '--format', 'json']
             with mock.patch('sys.argv', testargs):
-                generator.upgrade_policy()
+                generator.upgrade_policy(conf=self.local_conf)
                 new_file = self.get_config_file_fullname('new_policy.json')
                 new_policy = jsonutils.loads(open(new_file, 'r').read())
                 self.assertIsNotNone(new_policy.get('new_policy_name'))
@@ -635,7 +691,7 @@ class UpgradePolicyTestCase(base.PolicyBaseTestCase):
                         self.get_config_file_fullname('new_policy.yaml'),
                         '--format', 'yaml']
             with mock.patch('sys.argv', testargs):
-                generator.upgrade_policy()
+                generator.upgrade_policy(conf=self.local_conf)
                 new_file = self.get_config_file_fullname('new_policy.yaml')
                 new_policy = yaml.safe_load(open(new_file, 'r'))
                 self.assertIsNotNone(new_policy.get('new_policy_name'))
@@ -653,7 +709,7 @@ class UpgradePolicyTestCase(base.PolicyBaseTestCase):
                         '--namespace', 'test_upgrade',
                         '--format', 'json']
             with mock.patch('sys.argv', testargs):
-                generator.upgrade_policy()
+                generator.upgrade_policy(conf=self.local_conf)
                 expected = '''{
     "new_policy_name": "rule:admin"
 }'''
@@ -671,7 +727,7 @@ class UpgradePolicyTestCase(base.PolicyBaseTestCase):
                         '--namespace', 'test_upgrade',
                         '--format', 'yaml']
             with mock.patch('sys.argv', testargs):
-                generator.upgrade_policy()
+                generator.upgrade_policy(conf=self.local_conf)
                 expected = '''new_policy_name: rule:admin
 '''
                 self.assertEqual(expected, stdout.getvalue())
