@@ -19,6 +19,7 @@ import os
 
 import mock
 from oslo_config import cfg
+from oslo_context import context
 from oslo_serialization import jsonutils
 from oslotest import base as test_base
 import six
@@ -120,8 +121,7 @@ class RulesTestCase(test_base.BaseTestCase):
 
     @mock.patch.object(_parser, 'parse_rule', lambda x: x)
     def test_load_json_invalid_exc(self):
-        # When the JSON isn't valid, ValueError is raised on load_json.
-        # Note the trailing , in the exemplar is invalid JSON.
+        # When the JSON isn't valid, ValueError is raised on load.
         exemplar = """{
     "admin_or_owner": [["role:admin"], ["project_id:%(project_id)s"]],
     "default": [
@@ -129,6 +129,38 @@ class RulesTestCase(test_base.BaseTestCase):
         self.assertRaises(ValueError, policy.Rules.load, exemplar,
                           'default')
 
+        # However, since change I43782d245d7652ba69613b26fe598ac79ec19929,
+        # policy.Rules.load() first tries loading with the really fast
+        # jsonutils.loads(), and if that fails, it tries loading with
+        # yaml.safe_load().  Since YAML is a superset of JSON, some strictly
+        # invalid JSON can be parsed correctly by policy.Rules.load() without
+        # raising an exception.  But that means that since 1.17.0, we've been
+        # accepting (strictly speaking) illegal JSON policy files, and for
+        # backward compatibility, we should continue to do so.  Thus the
+        # following are here to prevent regressions:
+
+        # JSON requires double quotes, but the YAML parser doesn't care
+        bad_but_acceptable = """{
+    'admin_or_owner': [["role:admin"], ["project_id:%(project_id)s"]],
+    'default': []
+}"""
+        self.assertTrue(policy.Rules.load(bad_but_acceptable, 'default'))
+
+        # JSON does not allow bare keys, but the YAML parser doesn't care
+        bad_but_acceptable = """{
+    admin_or_owner: [["role:admin"], ["project_id:%(project_id)s"]],
+    default: []
+}"""
+        self.assertTrue(policy.Rules.load(bad_but_acceptable, 'default'))
+
+        # JSON is picky about commas, but the YAML parser is more forgiving
+        # (Note the trailing , in the exemplar is invalid JSON.)
+        bad_but_acceptable = """{
+    admin_or_owner: [["role:admin"], ["project_id:%(project_id)s"]],
+    default: [],
+}"""
+        self.assertTrue(policy.Rules.load(bad_but_acceptable, 'default'))
+
     @mock.patch.object(_parser, 'parse_rule', lambda x: x)
     def test_load_empty_data(self):
         result = policy.Rules.load('', 'default')
@@ -154,8 +186,8 @@ default: []
 
     @mock.patch.object(_parser, 'parse_rule', lambda x: x)
     def test_load_yaml_invalid_exc(self):
-        # When the JSON isn't valid, ValueError is raised on load().
-        # Note the trailing , in the exemplar is invalid JSON.
+        # When the JSON is seriously invalid, ValueError is raised on load().
+        # (See test_load_json_invalid_exc for what 'seriously invalid' means.)
         exemplar = """{
 # Define a custom rule.
 admin_or_owner: role:admin or project_id:%(project_id)s
@@ -390,6 +422,66 @@ class EnforcerTest(base.PolicyBaseTestCase):
                                group='oslo_policy')
         self.assertRaises(ValueError, self.enforcer.load_rules, True)
 
+    @mock.patch('oslo_policy.policy.Enforcer.check_rules')
+    def test_load_rules_twice(self, mock_check_rules):
+        self.enforcer.load_rules()
+        self.enforcer.load_rules()
+        self.assertEqual(1, mock_check_rules.call_count)
+
+    @mock.patch('oslo_policy.policy.Enforcer.check_rules')
+    def test_load_rules_twice_force(self, mock_check_rules):
+        self.enforcer.load_rules(True)
+        self.enforcer.load_rules(True)
+        self.assertEqual(2, mock_check_rules.call_count)
+
+    @mock.patch('oslo_policy.policy.Enforcer.check_rules')
+    def test_load_rules_twice_clear(self, mock_check_rules):
+        self.enforcer.load_rules()
+        self.enforcer.clear()
+        # NOTE(bnemec): It's weird that we have to pass True here, but clear
+        # sets enforcer.use_conf to False, which causes load_rules to be a
+        # noop when called with no parameters.  This is probably a bug.
+        self.enforcer.load_rules(True)
+        self.assertEqual(2, mock_check_rules.call_count)
+
+    @mock.patch('oslo_policy.policy.Enforcer.check_rules')
+    def test_load_directory_twice(self, mock_check_rules):
+        self.create_config_file(
+            os.path.join('policy.d', 'a.conf'), POLICY_A_CONTENTS)
+        self.create_config_file(
+            os.path.join('policy.d', 'b.conf'), POLICY_B_CONTENTS)
+        self.enforcer.load_rules()
+        self.enforcer.load_rules()
+        self.assertEqual(1, mock_check_rules.call_count)
+        self.assertIsNotNone(self.enforcer.rules)
+
+    @mock.patch('oslo_policy.policy.Enforcer.check_rules')
+    def test_load_directory_twice_force(self, mock_check_rules):
+        self.create_config_file(
+            os.path.join('policy.d', 'a.conf'), POLICY_A_CONTENTS)
+        self.create_config_file(
+            os.path.join('policy.d', 'b.conf'), POLICY_B_CONTENTS)
+        self.enforcer.load_rules(True)
+        self.enforcer.load_rules(True)
+        self.assertEqual(2, mock_check_rules.call_count)
+        self.assertIsNotNone(self.enforcer.rules)
+
+    @mock.patch('oslo_policy.policy.Enforcer.check_rules')
+    def test_load_directory_twice_changed(self, mock_check_rules):
+        self.create_config_file(
+            os.path.join('policy.d', 'a.conf'), POLICY_A_CONTENTS)
+        self.enforcer.load_rules()
+
+        # Touch the file
+        conf_path = os.path.join(self.config_dir, os.path.join(
+            'policy.d', 'a.conf'))
+        stinfo = os.stat(conf_path)
+        os.utime(conf_path, (stinfo.st_atime + 10, stinfo.st_mtime + 10))
+
+        self.enforcer.load_rules()
+        self.assertEqual(2, mock_check_rules.call_count)
+        self.assertIsNotNone(self.enforcer.rules)
+
     def test_set_rules_type(self):
         self.assertRaises(TypeError,
                           self.enforcer.set_rules,
@@ -646,6 +738,189 @@ class EnforcerTest(base.PolicyBaseTestCase):
                           self.enforcer.authorize, 'test', {},
                           {'roles': ['test']})
 
+    def test_enforcer_accepts_context_objects(self):
+        rule = policy.RuleDefault(name='fake_rule', check_str='role:test')
+        self.enforcer.register_default(rule)
+
+        request_context = context.RequestContext()
+        target_dict = {}
+        self.enforcer.enforce('fake_rule', target_dict, request_context)
+
+    def test_enforcer_accepts_subclassed_context_objects(self):
+        rule = policy.RuleDefault(name='fake_rule', check_str='role:test')
+        self.enforcer.register_default(rule)
+
+        class SpecializedContext(context.RequestContext):
+            pass
+
+        request_context = SpecializedContext()
+        target_dict = {}
+        self.enforcer.enforce('fake_rule', target_dict, request_context)
+
+    def test_enforcer_rejects_non_context_objects(self):
+        rule = policy.RuleDefault(name='fake_rule', check_str='role:test')
+        self.enforcer.register_default(rule)
+
+        class InvalidContext(object):
+            pass
+
+        request_context = InvalidContext()
+        target_dict = {}
+        self.assertRaises(
+            policy.InvalidContextObject, self.enforcer.enforce, 'fake_rule',
+            target_dict, request_context
+        )
+
+    @mock.patch.object(policy.Enforcer, '_map_context_attributes_into_creds')
+    def test_enforcer_call_map_context_attributes(self, map_mock):
+        map_mock.return_value = {}
+        rule = policy.RuleDefault(name='fake_rule', check_str='role:test')
+        self.enforcer.register_default(rule)
+
+        request_context = context.RequestContext()
+        target_dict = {}
+        self.enforcer.enforce('fake_rule', target_dict, request_context)
+        map_mock.assert_called_once_with(request_context)
+
+    def test_enforcer_consolidates_context_attributes_with_creds(self):
+        request_context = context.RequestContext()
+        expected_creds = request_context.to_policy_values()
+
+        creds = self.enforcer._map_context_attributes_into_creds(
+            request_context
+        )
+
+        # We don't use self.assertDictEqual here because to_policy_values
+        # actaully returns a non-dict object that just behaves like a
+        # dictionary, but does some special handling when people access
+        # deprecated policy values.
+        for k, v in expected_creds.items():
+            self.assertEqual(expected_creds[k], creds[k])
+
+    def test_map_context_attributes_populated_system(self):
+        request_context = context.RequestContext(system_scope='all')
+        expected_creds = request_context.to_policy_values()
+        expected_creds['system'] = 'all'
+
+        creds = self.enforcer._map_context_attributes_into_creds(
+            request_context
+        )
+
+        # We don't use self.assertDictEqual here because to_policy_values
+        # actaully returns a non-dict object that just behaves like a
+        # dictionary, but does some special handling when people access
+        # deprecated policy values.
+        for k, v in expected_creds.items():
+            self.assertEqual(expected_creds[k], creds[k])
+
+    def test_enforcer_accepts_policy_values_from_context(self):
+        rule = policy.RuleDefault(name='fake_rule', check_str='role:test')
+        self.enforcer.register_default(rule)
+
+        request_context = context.RequestContext()
+        policy_values = request_context.to_policy_values()
+        target_dict = {}
+        self.enforcer.enforce('fake_rule', target_dict, policy_values)
+
+    def test_enforcer_understands_system_scope(self):
+        self.conf.set_override('enforce_scope', True, group='oslo_policy')
+        rule = policy.RuleDefault(
+            name='fake_rule', check_str='role:test', scope_types=['system']
+        )
+        self.enforcer.register_default(rule)
+
+        ctx = context.RequestContext(system_scope='all')
+        target_dict = {}
+        self.enforcer.enforce('fake_rule', target_dict, ctx)
+
+    def test_enforcer_raises_invalid_scope_with_system_scope_type(self):
+        self.conf.set_override('enforce_scope', True, group='oslo_policy')
+        rule = policy.RuleDefault(
+            name='fake_rule', check_str='role:test', scope_types=['system']
+        )
+        self.enforcer.register_default(rule)
+
+        # model a domain-scoped token, which should fail enforcement
+        ctx = context.RequestContext(domain_id='fake')
+        target_dict = {}
+        self.assertRaises(
+            policy.InvalidScope, self.enforcer.enforce, 'fake_rule',
+            target_dict, ctx
+        )
+
+        # model a project-scoped token, which should fail enforcement
+        ctx = context.RequestContext(project_id='fake')
+        self.assertRaises(
+            policy.InvalidScope, self.enforcer.enforce, 'fake_rule',
+            target_dict, ctx
+        )
+
+    def test_enforcer_understands_domain_scope(self):
+        self.conf.set_override('enforce_scope', True, group='oslo_policy')
+        rule = policy.RuleDefault(
+            name='fake_rule', check_str='role:test', scope_types=['domain']
+        )
+        self.enforcer.register_default(rule)
+
+        ctx = context.RequestContext(domain_id='fake')
+        target_dict = {}
+        self.enforcer.enforce('fake_rule', target_dict, ctx)
+
+    def test_enforcer_raises_invalid_scope_with_domain_scope_type(self):
+        self.conf.set_override('enforce_scope', True, group='oslo_policy')
+        rule = policy.RuleDefault(
+            name='fake_rule', check_str='role:test', scope_types=['domain']
+        )
+        self.enforcer.register_default(rule)
+
+        # model a system-scoped token, which should fail enforcement
+        ctx = context.RequestContext(system_scope='all')
+        target_dict = {}
+        self.assertRaises(
+            policy.InvalidScope, self.enforcer.enforce, 'fake_rule',
+            target_dict, ctx
+        )
+
+        # model a project-scoped token, which should fail enforcement
+        ctx = context.RequestContext(project_id='fake')
+        self.assertRaises(
+            policy.InvalidScope, self.enforcer.enforce, 'fake_rule',
+            target_dict, ctx
+        )
+
+    def test_enforcer_understands_project_scope(self):
+        self.conf.set_override('enforce_scope', True, group='oslo_policy')
+        rule = policy.RuleDefault(
+            name='fake_rule', check_str='role:test', scope_types=['project']
+        )
+        self.enforcer.register_default(rule)
+
+        ctx = context.RequestContext(project_id='fake')
+        target_dict = {}
+        self.enforcer.enforce('fake_rule', target_dict, ctx)
+
+    def test_enforcer_raises_invalid_scope_with_project_scope_type(self):
+        self.conf.set_override('enforce_scope', True, group='oslo_policy')
+        rule = policy.RuleDefault(
+            name='fake_rule', check_str='role:test', scope_types=['project']
+        )
+        self.enforcer.register_default(rule)
+
+        # model a system-scoped token, which should fail enforcement
+        ctx = context.RequestContext(system_scope='all')
+        target_dict = {}
+        self.assertRaises(
+            policy.InvalidScope, self.enforcer.enforce, 'fake_rule',
+            target_dict, ctx
+        )
+
+        # model a domain-scoped token, which should fail enforcement
+        ctx = context.RequestContext(domain_id='fake')
+        self.assertRaises(
+            policy.InvalidScope, self.enforcer.enforce, 'fake_rule',
+            target_dict, ctx
+        )
+
 
 class EnforcerNoPolicyFileTest(base.PolicyBaseTestCase):
     def setUp(self):
@@ -697,15 +972,17 @@ class CheckFunctionTestCase(base.PolicyBaseTestCase):
 
     def test_check_explicit(self):
         rule = base.FakeCheck()
-        result = self.enforcer.enforce(rule, 'target', 'creds')
-        self.assertEqual(('target', 'creds', self.enforcer), result)
+        creds = {}
+        result = self.enforcer.enforce(rule, 'target', creds)
+        self.assertEqual(('target', creds, self.enforcer), result)
 
     def test_check_no_rules(self):
         # Clear the policy.json file created in setUp()
         self.create_config_file('policy.json', "{}")
         self.enforcer.default_rule = None
         self.enforcer.load_rules()
-        result = self.enforcer.enforce('rule', 'target', 'creds')
+        creds = {}
+        result = self.enforcer.enforce('rule', 'target', creds)
         self.assertFalse(result)
 
     def test_check_with_rule(self):
@@ -722,7 +999,8 @@ class CheckFunctionTestCase(base.PolicyBaseTestCase):
         self.create_config_file('policy.json', jsonutils.dumps({"a_rule": []}))
         self.enforcer.default_rule = None
         self.enforcer.load_rules()
-        result = self.enforcer.enforce('rule', 'target', 'creds')
+        creds = {}
+        result = self.enforcer.enforce('rule', 'target', creds)
         self.assertFalse(result)
 
     def test_check_raise_default(self):
@@ -742,7 +1020,8 @@ class CheckFunctionTestCase(base.PolicyBaseTestCase):
         creds = {}
         exc = self.assertRaises(
             MyException, self.enforcer.enforce, 'rule', 'target', creds,
-            True, MyException, 'arg1', 'arg2', kw1='kwarg1', kw2='kwarg2')
+            True, MyException, 'arg1', 'arg2', kw1='kwarg1',
+            kw2='kwarg2')
         self.assertEqual(('arg1', 'arg2'), exc.args)
         self.assertEqual(dict(kw1='kwarg1', kw2='kwarg2'), exc.kwargs)
 
@@ -843,6 +1122,42 @@ class RuleDefaultTestCase(base.PolicyBaseTestCase):
         opt2 = RuleDefaultSub(name='bar', check_str='rule:foo')
         self.assertNotEqual(opt1, opt2)
 
+    def test_create_opt_with_scope_types(self):
+        scope_types = ['project']
+        opt = policy.RuleDefault(
+            name='foo',
+            check_str='role:bar',
+            scope_types=scope_types
+        )
+        self.assertEqual(opt.scope_types, scope_types)
+
+    def test_create_opt_with_scope_type_strings_fails(self):
+        self.assertRaises(
+            ValueError,
+            policy.RuleDefault,
+            name='foo',
+            check_str='role:bar',
+            scope_types='project'
+        )
+
+    def test_create_opt_with_multiple_scope_types(self):
+        opt = policy.RuleDefault(
+            name='foo',
+            check_str='role:bar',
+            scope_types=['project', 'domain', 'system']
+        )
+
+        self.assertEqual(opt.scope_types, ['project', 'domain', 'system'])
+
+    def test_ensure_scope_types_are_unique(self):
+        self.assertRaises(
+            ValueError,
+            policy.RuleDefault,
+            name='foo',
+            check_str='role:bar',
+            scope_types=['project', 'project']
+        )
+
 
 class DocumentedRuleDefaultDeprecationTestCase(base.PolicyBaseTestCase):
 
@@ -875,6 +1190,70 @@ class DocumentedRuleDefaultDeprecationTestCase(base.PolicyBaseTestCase):
             enforcer.load_rules()
             mock_warn.assert_called_once_with(expected_msg)
 
+        self.assertTrue(
+            enforcer.enforce('foo:create_bar', {}, {'roles': ['bang']})
+        )
+        self.assertTrue(
+            enforcer.enforce('foo:create_bar', {}, {'roles': ['fizz']})
+        )
+        self.assertFalse(
+            enforcer.enforce('foo:create_bar', {}, {'roles': ['baz']})
+        )
+
+    def test_deprecate_an_empty_policy_check_string(self):
+        deprecated_rule = policy.DeprecatedRule(
+            name='foo:create_bar',
+            check_str=''
+        )
+
+        rule_list = [policy.DocumentedRuleDefault(
+            name='foo:create_bar',
+            check_str='role:bang',
+            description='Create a bar.',
+            operations=[{'path': '/v1/bars', 'method': 'POST'}],
+            deprecated_rule=deprecated_rule,
+            deprecated_reason='because of reasons',
+            deprecated_since='N'
+        )]
+        enforcer = policy.Enforcer(self.conf)
+        enforcer.register_defaults(rule_list)
+
+        with mock.patch('warnings.warn') as mock_warn:
+            enforcer.load_rules()
+            mock_warn.assert_called_once()
+
+        enforcer.enforce('foo:create_bar', {}, {'roles': ['bang']},
+                         do_raise=True)
+        enforcer.enforce('foo:create_bar', {}, {'roles': ['fizz']},
+                         do_raise=True)
+
+    def test_deprecate_replace_with_empty_policy_check_string(self):
+        deprecated_rule = policy.DeprecatedRule(
+            name='foo:create_bar',
+            check_str='role:fizz'
+        )
+
+        rule_list = [policy.DocumentedRuleDefault(
+            name='foo:create_bar',
+            check_str='',
+            description='Create a bar.',
+            operations=[{'path': '/v1/bars', 'method': 'POST'}],
+            deprecated_rule=deprecated_rule,
+            deprecated_reason='because of reasons',
+            deprecated_since='N'
+        )]
+        enforcer = policy.Enforcer(self.conf)
+        enforcer.register_defaults(rule_list)
+
+        with mock.patch('warnings.warn') as mock_warn:
+            enforcer.load_rules()
+            mock_warn.assert_called_once()
+
+        enforcer.enforce('foo:create_bar', {}, {'roles': ['fizz']},
+                         do_raise=True)
+        enforcer.enforce('foo:create_bar', {}, {'roles': ['bang']},
+                         do_raise=True)
+
     def test_deprecate_a_policy_name(self):
         deprecated_rule = policy.DeprecatedRule(
             name='foo:bar',
@@ -919,7 +1298,7 @@ class DocumentedRuleDefaultDeprecationTestCase(base.PolicyBaseTestCase):
             enforcer.load_rules(True)
             mock_warn.assert_called_once_with(expected_msg)
 
-    def test_deprecate_a_policy_for_removal(self):
+    def test_deprecate_a_policy_for_removal_logs_warning_when_overridden(self):
         rule_list = [policy.DocumentedRuleDefault(
             name='foo:bar',
             check_str='role:baz',
@@ -945,6 +1324,97 @@ class DocumentedRuleDefaultDeprecationTestCase(base.PolicyBaseTestCase):
             enforcer.load_rules()
             mock_warn.assert_called_once_with(expected_msg)
 
+    def test_deprecate_a_policy_for_removal_does_not_log_warning(self):
+        # We should only log a warning for operators if they are supplying an
+        # override for a policy that is deprecated for removal.
+        rule_list = [policy.DocumentedRuleDefault(
+            name='foo:bar',
+            check_str='role:baz',
+            description='Create a foo.',
+            operations=[{'path': '/v1/foos/', 'method': 'POST'}],
+            deprecated_for_removal=True,
+            deprecated_reason=(
+                '"foo:bar" is no longer a policy used by the service'
+            ),
+            deprecated_since='N'
+        )]
+        enforcer = policy.Enforcer(self.conf)
+        enforcer.register_defaults(rule_list)
+
+        with mock.patch('warnings.warn') as mock_warn:
+            enforcer.load_rules()
+            mock_warn.assert_not_called()
+
+    def test_deprecate_check_str_suppress_does_not_log_warning(self):
+        deprecated_rule = policy.DeprecatedRule(
+            name='foo:create_bar',
+            check_str='role:fizz'
+        )
+
+        rule_list = [policy.DocumentedRuleDefault(
+            name='foo:create_bar',
+            check_str='role:bang',
+            description='Create a bar.',
+            operations=[{'path': '/v1/bars', 'method': 'POST'}],
+            deprecated_rule=deprecated_rule,
+            deprecated_reason='"role:bang" is a better default',
+            deprecated_since='N'
+        )]
+        enforcer = policy.Enforcer(self.conf)
+        enforcer.suppress_deprecation_warnings = True
+        enforcer.register_defaults(rule_list)
+        with mock.patch('warnings.warn') as mock_warn:
+            enforcer.load_rules()
+            mock_warn.assert_not_called()
+
+    def test_deprecate_name_suppress_does_not_log_warning(self):
+        deprecated_rule = policy.DeprecatedRule(
+            name='foo:bar',
+            check_str='role:baz'
+        )
+
+        rule_list = [policy.DocumentedRuleDefault(
+            name='foo:create_bar',
+            check_str='role:baz',
+            description='Create a bar.',
+            operations=[{'path': '/v1/bars/', 'method': 'POST'}],
+            deprecated_rule=deprecated_rule,
+            deprecated_reason='"foo:bar" is not granular enough.',
+            deprecated_since='N'
+        )]
+
+        rules = jsonutils.dumps({'foo:bar': 'role:bang'})
+        self.create_config_file('policy.json', rules)
+        enforcer = policy.Enforcer(self.conf)
+        enforcer.suppress_deprecation_warnings = True
+        enforcer.register_defaults(rule_list)
+
+        with mock.patch('warnings.warn') as mock_warn:
+            enforcer.load_rules()
+            mock_warn.assert_not_called()
+
+    def test_deprecate_for_removal_suppress_does_not_log_warning(self):
+        rule_list = [policy.DocumentedRuleDefault(
+            name='foo:bar',
+            check_str='role:baz',
+            description='Create a foo.',
+            operations=[{'path': '/v1/foos/', 'method': 'POST'}],
+            deprecated_for_removal=True,
+            deprecated_reason=(
+                '"foo:bar" is no longer a policy used by the service'
+            ),
+            deprecated_since='N'
+        )]
+        rules = jsonutils.dumps({'foo:bar': 'role:bang'})
+        self.create_config_file('policy.json', rules)
+        enforcer = policy.Enforcer(self.conf)
+        enforcer.suppress_deprecation_warnings = True
+        enforcer.register_defaults(rule_list)
+
+        with mock.patch('warnings.warn') as mock_warn:
+            enforcer.load_rules()
+            mock_warn.assert_not_called()
+
     def test_deprecated_policy_for_removal_must_include_deprecated_since(self):
         self.assertRaises(
             ValueError,
@@ -998,6 +1468,157 @@ class DocumentedRuleDefaultDeprecationTestCase(base.PolicyBaseTestCase):
             deprecated_since='N'
         )
 
+    def test_override_deprecated_policy_with_old_name(self):
+        # Simulate an operator overriding a policy
+        rules = jsonutils.dumps({'foo:bar': 'role:bazz'})
+        self.create_config_file('policy.json', rules)
+
+        # Deprecate the policy name and check string in favor of something
+        # better.
+        deprecated_rule = policy.DeprecatedRule(
+            name='foo:bar',
+            check_str='role:fizz'
+        )
+        rule_list = [policy.DocumentedRuleDefault(
+            name='foo:create_bar',
+            check_str='role:bang',
+            description='Create a bar.',
+            operations=[{'path': '/v1/bars', 'method': 'POST'}],
+            deprecated_rule=deprecated_rule,
+            deprecated_reason='"role:bang" is a better default',
+            deprecated_since='N'
+        )]
+        self.enforcer.register_defaults(rule_list)
+
+        # Make sure the override supplied by the operator using the old policy
+        # name is used in favor of the old or new default.
+        self.assertFalse(
+            self.enforcer.enforce('foo:create_bar', {}, {'roles': ['fizz']})
+        )
+        self.assertFalse(
+            self.enforcer.enforce('foo:create_bar', {}, {'roles': ['bang']})
+        )
+        self.assertTrue(
+            self.enforcer.enforce('foo:create_bar', {}, {'roles': ['bazz']})
+        )
+
+    def test_override_deprecated_policy_with_new_name(self):
+        # Simulate an operator overriding a policy using the new policy name
+        rules = jsonutils.dumps({'foo:create_bar': 'role:bazz'})
+        self.create_config_file('policy.json', rules)
+
+        # Deprecate the policy name and check string in favor of something
+        # better.
+        deprecated_rule = policy.DeprecatedRule(
+            name='foo:bar',
+            check_str='role:fizz'
+        )
+        rule_list = [policy.DocumentedRuleDefault(
+            name='foo:create_bar',
+            check_str='role:bang',
+            description='Create a bar.',
+            operations=[{'path': '/v1/bars', 'method': 'POST'}],
+            deprecated_rule=deprecated_rule,
+            deprecated_reason='"role:bang" is a better default',
+            deprecated_since='N'
+        )]
+        self.enforcer.register_defaults(rule_list)
+
+        # Make sure the override supplied by the operator is being used in
+        # place of either default value.
+        self.assertFalse(
+            self.enforcer.enforce('foo:create_bar', {}, {'roles': ['fizz']})
+        )
+        self.assertFalse(
+            self.enforcer.enforce('foo:create_bar', {}, {'roles': ['bang']})
+        )
+        self.assertTrue(
+            self.enforcer.enforce('foo:create_bar', {}, {'roles': ['bazz']})
+        )
+
+    def test_override_both_new_and_old_policy(self):
+        # Simulate an operator overriding a policy using both the the new and
+        # old policy names. The following doesn't make a whole lot of sense
+        # because the overrides are conflicting, but we want to make sure that
+        # oslo.policy uses foo:create_bar instead of foo:bar.
+        rules_dict = {
+            'foo:create_bar': 'role:bazz',
+            'foo:bar': 'role:wee'
+        }
+        rules = jsonutils.dumps(rules_dict)
+        self.create_config_file('policy.json', rules)
+
+        # Deprecate the policy name and check string in favor of something
+        # better.
+        deprecated_rule = policy.DeprecatedRule(
+            name='foo:bar',
+            check_str='role:fizz'
+        )
+        rule_list = [policy.DocumentedRuleDefault(
+            name='foo:create_bar',
+            check_str='role:bang',
+            description='Create a bar.',
+            operations=[{'path': '/v1/bars', 'method': 'POST'}],
+            deprecated_rule=deprecated_rule,
+            deprecated_reason='"role:bang" is a better default',
+            deprecated_since='N'
+        )]
+        self.enforcer.register_defaults(rule_list)
+
+        # The default check string for the old policy name foo:bar should fail
+        self.assertFalse(
+            self.enforcer.enforce('foo:create_bar', {}, {'roles': ['fizz']})
+        )
+
+        # The default check string for the new policy name foo:create_bar
+        # should fail
+        self.assertFalse(
+            self.enforcer.enforce('foo:create_bar', {}, {'roles': ['bang']})
+        )
+
+        # The override for the old policy name foo:bar should fail
+        self.assertFalse(
+            self.enforcer.enforce('foo:create_bar', {}, {'roles': ['wee']})
+        )
+
+        # The override for foo:create_bar should pass
+        self.assertTrue(
+            self.enforcer.enforce('foo:create_bar', {}, {'roles': ['bazz']})
+        )
+
+    def test_override_deprecated_policy_with_new_rule(self):
+        # Simulate an operator overriding a deprecated policy with a reference
+        # to the new policy, as done by the sample policy generator.
+        rules = jsonutils.dumps({'old_rule': 'rule:new_rule'})
+        self.create_config_file('policy.json', rules)
+
+        # Deprecate the policy name in favor of something better.
+        deprecated_rule = policy.DeprecatedRule(
+            name='old_rule',
+            check_str='role:bang'
+        )
+        rule_list = [policy.DocumentedRuleDefault(
+            name='new_rule',
+            check_str='role:bang',
+            description='Replacement for old_rule.',
+            operations=[{'path': '/v1/bars', 'method': 'POST'}],
+            deprecated_rule=deprecated_rule,
+            deprecated_reason='"old_rule" is a bad name',
+            deprecated_since='N'
+        )]
+        self.enforcer.register_defaults(rule_list)
+
+        # Make sure the override supplied by the operator using the old policy
+        # name is used in favor of the old or new default.
+        self.assertFalse(
+            self.enforcer.enforce('new_rule', {}, {'roles': ['fizz']})
+        )
+        self.assertTrue(
+            self.enforcer.enforce('new_rule', {}, {'roles': ['bang']})
+        )
+        # Verify that we didn't overwrite the new rule.
+        self.assertEqual('bang', self.enforcer.rules['new_rule'].match)
+
 
 class DocumentedRuleDefaultTestCase(base.PolicyBaseTestCase):
 
